import { Rule, RuleOptions } from './Rule';
import { RawRuleFrom } from './RawRule';
import { CanParameters, Abilities, Normalize, SubjectType } from './types';
import { LinkedItem } from './structures/LinkedItem';
export interface RuleIndexOptions<A extends Abilities, C> extends Partial<RuleOptions<A, C>> {
    detectSubjectType?(subject?: Normalize<A>[1]): string;
}
declare const $abilities: unique symbol;
declare const $conditions: unique symbol;
interface WithGenerics {
    [$abilities]: any;
    [$conditions]: any;
}
export declare type Public<T extends WithGenerics> = {
    [K in keyof T]: T[K];
};
export declare type Generics<T extends WithGenerics> = {
    abilities: T[typeof $abilities];
    conditions: T[typeof $conditions];
};
export declare type RuleOf<T extends WithGenerics> = Rule<Generics<T>['abilities'], Generics<T>['conditions']>;
export declare type RawRuleOf<T extends WithGenerics> = RawRuleFrom<Generics<T>['abilities'], Generics<T>['conditions']>;
export declare type RuleIndexOptionsOf<T extends WithGenerics> = RuleIndexOptions<Generics<T>['abilities'], Generics<T>['conditions']>;
export interface UpdateEvent<T extends WithGenerics> {
    rules: RawRuleOf<T>[];
}
export declare type EventHandler<Event> = (event: Event) => void;
export declare type Events<T extends WithGenerics, Event extends {} = {}, K extends keyof EventsMap<T, Event> = keyof EventsMap<T, Event>> = Map<K, {
    last: LinkedItem<EventHandler<EventsMap<T, Event>[K]>> | null;
    destroy: Array<Unsubscribe> | null;
    emits: boolean;
}>;
interface EventsMap<T extends WithGenerics, Event extends {} = {}> {
    update: UpdateEvent<T> & Event;
    updated: UpdateEvent<T> & Event;
}
export declare type Unsubscribe = () => void;
export declare class RuleIndex<A extends Abilities, Conditions, BaseEvent extends {} = {}> {
    private _hasPerFieldRules;
    private _events;
    private _indexedRules;
    private _rules;
    private readonly _ruleOptions;
    readonly detectSubjectType: Exclude<RuleIndexOptions<A, Conditions>['detectSubjectType'], undefined>;
    readonly [$abilities]: A;
    readonly [$conditions]: Conditions;
    constructor(rules?: RawRuleFrom<A, Conditions>[], options?: RuleIndexOptions<A, Conditions>);
    get rules(): (import("./types").ToAbilityTypes<A> extends import("./types").AbilityTupleType<string, SubjectType> ? import("./RawRule").SubjectRawRule<(import("./types").AbilityTupleType<string, SubjectType> & import("./types").ToAbilityTypes<A>)[0], (import("./types").AbilityTupleType<string, SubjectType> & import("./types").ToAbilityTypes<A>)[1], Conditions> : import("./RawRule").ClaimRawRule<Extract<import("./types").ToAbilityTypes<A>, string>>)[];
    update(rules: RawRuleFrom<A, Conditions>[]): Public<this>;
    private _buildIndexFor;
    private _analyze;
    possibleRulesFor(...[action, subject]: CanParameters<A, false>): Rule<A, Conditions>[];
    rulesFor(...args: CanParameters<A>): Rule<A, Conditions>[];
    on<T extends keyof EventsMap<this, BaseEvent>>(event: T, handler: EventHandler<EventsMap<this, BaseEvent>[T]>): Unsubscribe;
    private _emit;
}
export {};
